/** @jsx jsx */
import { css, jsx } from '@emotion/react';
import invariant from 'tiny-invariant';
import { colorMap } from '../internal/color-map';
import { useSurface } from './surface-provider';
const asAllowlist = ['span', 'div', 'p'];
const fontFamily = `-apple-system, BlinkMacSystemFont, 'Segoe UI', 'Roboto', 'Oxygen', 'Ubuntu', 'Fira Sans', 'Droid Sans', 'Helvetica Neue', sans-serif`;
const fontSizeMap = {
  '11px': css({
    fontSize: '11px'
  }),
  '12px': css({
    fontSize: '12px'
  }),
  '14px': css({
    fontSize: '14px'
  })
};
// NOTE: can't use numbers as keys or Constellation won't build. Weird one.
const fontWeightMap = {
  '400': css({
    fontWeight: 400
  }),
  '500': css({
    fontWeight: 500
  }),
  '700': css({
    fontWeight: 700
  })
};
const lineHeightMap = {
  '12px': css({
    lineHeight: '12px'
  }),
  '16px': css({
    lineHeight: '16px'
  }),
  '20px': css({
    lineHeight: '20px'
  }),
  '24px': css({
    lineHeight: '24px'
  }),
  '28px': css({
    lineHeight: '28px'
  }),
  '32px': css({
    lineHeight: '32px'
  }),
  '40px': css({
    lineHeight: '40px'
  })
};
const textAlignMap = {
  center: css({
    textAlign: 'center'
  }),
  end: css({
    textAlign: 'end'
  }),
  start: css({
    textAlign: 'start'
  })
};
const textTransformMap = {
  none: css({
    textTransform: 'none'
  }),
  lowercase: css({
    textTransform: 'lowercase'
  }),
  uppercase: css({
    textTransform: 'uppercase'
  })
};
const verticalAlignMap = {
  top: css({
    verticalAlign: 'top'
  }),
  middle: css({
    verticalAlign: 'middle'
  }),
  bottom: css({
    verticalAlign: 'bottom'
  })
};
const baseStyles = css({
  boxSizing: 'border-box',
  margin: '0px',
  padding: '0px',
  fontFamily
});
const truncateStyles = css({
  overflow: 'hidden',
  textOverflow: 'ellipsis',
  whiteSpace: 'nowrap'
});
/**
 * __Text__
 *
 * Text is a primitive component that has the Atlassian Design System's design guidelines baked in.
 * This includes considerations for text attributes such as color, font size, font weight, and line height.
 * It renders a `span` by default.
 *
 * @internal
 */

const Text = ({
  as: Component = 'span',
  children,
  color: colorTuple,
  fontSize,
  fontWeight,
  lineHeight,
  shouldTruncate = false,
  textAlign,
  textTransform,
  verticalAlign,
  testId,
  UNSAFE_style
}) => {
  const surface = useSurface(); // @ts-ignore

  const [color = colorMap[surface], fallback] = colorTuple || [];
  invariant(asAllowlist.includes(Component), `@atlaskit/ds-explorations: Text received an invalid "as" value of "${Component}"`);
  return jsx(Component, {
    style: { ...UNSAFE_style,
      ...(fallback && {
        '--ds-co-fb': fallback
      })
    },
    css: [baseStyles, color && textColorMap[color], fontSize && fontSizeMap[fontSize], fontWeight && fontWeightMap[fontWeight], lineHeight && lineHeightMap[lineHeight], shouldTruncate && truncateStyles, textAlign && textAlignMap[textAlign], textTransform && textTransformMap[textTransform], verticalAlign && verticalAlignMap[verticalAlign]],
    "data-testid": testId
  }, children);
};

export default Text;
/**
 * THIS SECTION WAS CREATED VIA CODEGEN DO NOT MODIFY {@see http://go/af-codegen}
 * @codegen <<SignedSource::140ffff6e1310c1c37e2067e2c232b92>>
 * @codegenId colors
 * @codegenCommand yarn codegen-styles
 * @codegenParams ["text"]
 */

const textColorMap = {
  'color.text': css({
    color: "var(--ds-text, var(--ds-co-fb))"
  }),
  subtle: css({
    color: "var(--ds-text-subtle, var(--ds-co-fb))"
  }),
  subtlest: css({
    color: "var(--ds-text-subtlest, var(--ds-co-fb))"
  }),
  disabled: css({
    color: "var(--ds-text-disabled, var(--ds-co-fb))"
  }),
  inverse: css({
    color: "var(--ds-text-inverse, var(--ds-co-fb))"
  }),
  brand: css({
    color: "var(--ds-text-brand, var(--ds-co-fb))"
  }),
  selected: css({
    color: "var(--ds-text-selected, var(--ds-co-fb))"
  }),
  danger: css({
    color: "var(--ds-text-danger, var(--ds-co-fb))"
  }),
  warning: css({
    color: "var(--ds-text-warning, var(--ds-co-fb))"
  }),
  'warning.inverse': css({
    color: "var(--ds-text-warning-inverse, var(--ds-co-fb))"
  }),
  success: css({
    color: "var(--ds-text-success, var(--ds-co-fb))"
  }),
  discovery: css({
    color: "var(--ds-text-discovery, var(--ds-co-fb))"
  }),
  information: css({
    color: "var(--ds-text-information, var(--ds-co-fb))"
  })
};
/**
 * @codegenEnd
 */