/** @jsx jsx */
import { ElementType, HTMLAttributes } from 'react';
import { GlobalSpacingToken } from '../constants';
import type { BasePrimitiveProps, NonTextChildren } from './types';
export interface BoxProps<T extends HTMLElement = HTMLElement> extends Omit<HTMLAttributes<T>, 'style' | 'as' | 'className'>, BasePrimitiveProps {
    /**
     * The DOM element to render as the Box. Defaults to `div`.
     */
    as?: ElementType;
    /**
     * Elements to be rendered inside the Box.
     */
    children?: NonTextChildren | boolean | null | undefined;
    /**
     * The html className attribute.
     *
     * Before using this prop please ensure:
     * - The styles cannot otherwise be achieved through `Box` directly.
     * - The use case needs custom styles that cannot be designed or implemented differently
     *
     * Ensure you're using the `@atlaskit/eslint-plugin-design-system` with this prop to prevent unbounded usage.
     *
     * @see `@atlaskit/eslint-plugin-design-system`
     */
    className?: string;
    /**
     * Token representing background color with a fallback.
     */
    backgroundColor?: [BackgroundColor, string];
    /**
     * Defines border style.
     */
    borderStyle?: BorderStyle;
    /**
     * Defines border width.
     */
    borderWidth?: BorderWidth;
    /**
     * Token representing border color with a fallback.
     */
    borderColor?: [BorderColor, string];
    /**
     * Defines border radius.
     */
    borderRadius?: BorderRadius;
    /**
     * Defines the main axis direction.
     */
    flexDirection?: FlexDirection;
    /**
     * Used to align children along the cross axis.
     */
    alignItems?: FlexAlignItems;
    /**
     * Used to align children along the main axis.
     */
    justifyContent?: FlexJustifyContent;
    /**
     * Defines what happens if content overflows the box.
     */
    overflow?: Overflow;
    /**
     * Shorthand for `paddingBlock` and `paddingInline` together.
     *
     * @see paddingBlock
     * @see paddingInline
     */
    padding?: GlobalSpacingToken;
    /**
     * Token representing CSS `padding-block`.
     */
    paddingBlock?: GlobalSpacingToken;
    /**
     * Token representing CSS `padding-inline`.
     */
    paddingInline?: GlobalSpacingToken;
    /**
     * Token representing width.
     */
    width?: GlobalSpacingToken;
    /**
     * Token representing height.
     */
    height?: GlobalSpacingToken;
    /**
     * Defines display type and layout. Defaults to `flex`.
     */
    display?: Display;
    /**
     * CSS position property.
     */
    position?: keyof typeof positionMap;
}
/**
 * __Box__
 *
 * Box is a primitive component that has the design decisions of the Atlassian Design System baked in.
 * Renders a `div` by default.
 *
 * @internal
 */
declare const Box: import("react").ForwardRefExoticComponent<BoxProps<HTMLElement> & import("react").RefAttributes<HTMLElement>>;
export default Box;
declare type BorderStyle = keyof typeof borderStyleMap;
declare const borderStyleMap: {
    none: import("@emotion/react").SerializedStyles;
    solid: import("@emotion/react").SerializedStyles;
    dashed: import("@emotion/react").SerializedStyles;
    dotted: import("@emotion/react").SerializedStyles;
};
declare type BorderWidth = keyof typeof borderWidthMap;
declare const borderWidthMap: {
    '0px': import("@emotion/react").SerializedStyles;
    '1px': import("@emotion/react").SerializedStyles;
    '2px': import("@emotion/react").SerializedStyles;
    '3px': import("@emotion/react").SerializedStyles;
};
declare type BorderRadius = keyof typeof borderRadiusMap;
declare const borderRadiusMap: {
    normal: import("@emotion/react").SerializedStyles;
    rounded: import("@emotion/react").SerializedStyles;
    badge: import("@emotion/react").SerializedStyles;
};
declare type FlexDirection = keyof typeof flexDirectionMap;
declare const flexDirectionMap: {
    column: import("@emotion/react").SerializedStyles;
    row: import("@emotion/react").SerializedStyles;
};
declare type FlexAlignItems = keyof typeof flexAlignItemsMap;
declare const flexAlignItemsMap: {
    center: import("@emotion/react").SerializedStyles;
    baseline: import("@emotion/react").SerializedStyles;
    flexStart: import("@emotion/react").SerializedStyles;
    flexEnd: import("@emotion/react").SerializedStyles;
};
declare type FlexJustifyContent = keyof typeof flexJustifyContentMap;
declare const flexJustifyContentMap: {
    center: import("@emotion/react").SerializedStyles;
    flexStart: import("@emotion/react").SerializedStyles;
    flexEnd: import("@emotion/react").SerializedStyles;
};
declare type Display = keyof typeof displayMap;
declare const displayMap: {
    block: import("@emotion/react").SerializedStyles;
    inline: import("@emotion/react").SerializedStyles;
    flex: import("@emotion/react").SerializedStyles;
    inlineFlex: import("@emotion/react").SerializedStyles;
};
declare type Overflow = keyof typeof overflowMap;
declare const overflowMap: {
    auto: import("@emotion/react").SerializedStyles;
};
declare const positionMap: {
    absolute: import("@emotion/react").SerializedStyles;
    relative: import("@emotion/react").SerializedStyles;
    static: import("@emotion/react").SerializedStyles;
};
/**
 * @codegenEnd
 */
/**
 * THIS SECTION WAS CREATED VIA CODEGEN DO NOT MODIFY {@see http://go/af-codegen}
 * @codegen <<SignedSource::ebb55786a54803214357d0eef0cac448>>
 * @codegenId colors
 * @codegenCommand yarn codegen-styles
 * @codegenParams ["border", "background"]
 */
declare const borderColorMap: {
    'color.border': import("@emotion/react").SerializedStyles;
    bold: import("@emotion/react").SerializedStyles;
    inverse: import("@emotion/react").SerializedStyles;
    focused: import("@emotion/react").SerializedStyles;
    input: import("@emotion/react").SerializedStyles;
    disabled: import("@emotion/react").SerializedStyles;
    brand: import("@emotion/react").SerializedStyles;
    selected: import("@emotion/react").SerializedStyles;
    danger: import("@emotion/react").SerializedStyles;
    warning: import("@emotion/react").SerializedStyles;
    success: import("@emotion/react").SerializedStyles;
    discovery: import("@emotion/react").SerializedStyles;
    information: import("@emotion/react").SerializedStyles;
};
export declare type BorderColor = keyof typeof borderColorMap;
declare const backgroundColorMap: {
    disabled: import("@emotion/react").SerializedStyles;
    'inverse.subtle': import("@emotion/react").SerializedStyles;
    input: import("@emotion/react").SerializedStyles;
    neutral: import("@emotion/react").SerializedStyles;
    'neutral.subtle': import("@emotion/react").SerializedStyles;
    'neutral.bold': import("@emotion/react").SerializedStyles;
    'brand.bold': import("@emotion/react").SerializedStyles;
    selected: import("@emotion/react").SerializedStyles;
    'selected.bold': import("@emotion/react").SerializedStyles;
    danger: import("@emotion/react").SerializedStyles;
    'danger.bold': import("@emotion/react").SerializedStyles;
    warning: import("@emotion/react").SerializedStyles;
    'warning.bold': import("@emotion/react").SerializedStyles;
    success: import("@emotion/react").SerializedStyles;
    'success.bold': import("@emotion/react").SerializedStyles;
    discovery: import("@emotion/react").SerializedStyles;
    'discovery.bold': import("@emotion/react").SerializedStyles;
    information: import("@emotion/react").SerializedStyles;
    'information.bold': import("@emotion/react").SerializedStyles;
    'color.blanket': import("@emotion/react").SerializedStyles;
    'color.blanket.selected': import("@emotion/react").SerializedStyles;
    'color.blanket.danger': import("@emotion/react").SerializedStyles;
    'elevation.surface': import("@emotion/react").SerializedStyles;
    'elevation.surface.sunken': import("@emotion/react").SerializedStyles;
    'elevation.surface.raised': import("@emotion/react").SerializedStyles;
    'elevation.surface.overlay': import("@emotion/react").SerializedStyles;
};
export declare type BackgroundColor = keyof typeof backgroundColorMap;
/**
 * @codegenEnd
 */
